@@ -23,6 +23,7 @@ class CubeMeta(type):
                 cls._dimensions[name] = attr
                 if not attr.name:
                     attr.name = name
+                attr.cube = cls
             elif isinstance(attr, Metric):
                 cls._metrics[name] = attr
                 if not attr.name:
@@ -37,7 +38,8 @@ class Cube(metaclass=CubeMeta):
         :return:
         """
         return namedtuple(
-            cls.__name__ + "Record", list(cls._dimensions.keys()) + list(cls._metrics.keys())
+            cls.__name__ + "Record",
+            list(cls._dimensions.keys()) + list(cls._metrics.keys()),
         )
 
     @classmethod
@@ -71,8 +73,11 @@ class Cube(metaclass=CubeMeta):
         try:
             return cls._dimensions[name]
         except KeyError:
-            raise ConfigurationError(f"Unknown dimension: {name}")
+            raise ConfigurationError(f"Unknown dimension: {name}") from None
 
     @classmethod
     def metric_by_name(cls, name: str) -> Metric:
-        return cls._metrics[name]
+        try:
+            return cls._metrics[name]
+        except KeyError:
+            raise ConfigurationError(f"Unknown metric: {name}") from None
