@@ -1,10 +1,13 @@
 import abc
 import ipaddress
 from datetime import date, datetime
-from typing import Any, Optional, Union
+from typing import TYPE_CHECKING, Any, Optional, Type, Union
+
+if TYPE_CHECKING:
+    from hcube.api.models.cube import Cube
 
 
-class Dimension(abc.ABC):
+class Dimension(abc.ABC):  # noqa: B024
     """
     Dimensions define the actual axes of the data/cube.
     """
@@ -13,7 +16,11 @@ class Dimension(abc.ABC):
     default = None
 
     def __init__(
-        self, name: Optional[str] = None, null: bool = False, help_text: str = "", **kwargs
+        self,
+        name: Optional[str] = None,
+        null: bool = False,
+        help_text: str = "",
+        **kwargs,
     ):
         """
         Name could be supplied later on if the dimension is declared in HCube class specification
@@ -29,6 +36,11 @@ class Dimension(abc.ABC):
         self.null = null
         self.help_text = help_text
         self.kwargs = kwargs
+        self.cube: Optional[Type[Cube]] = None
+
+    def __str__(self):
+        cube_name = self.cube.__name__ if self.cube else "UnknownCube"
+        return f"{cube_name}.{self.name}"
 
     def to_python(self, value: Any):
         """
@@ -53,19 +65,16 @@ class Dimension(abc.ABC):
 
 
 class StringDimension(Dimension):
-
     _type = str
     default = ""
 
 
 class BooleanDimension(Dimension):
-
     _type = bool
     default = False
 
 
 class IntDimension(Dimension):
-
     _type = int
     default = 0
 
@@ -88,7 +97,6 @@ class IntDimension(Dimension):
 
 
 class DateDimension(Dimension):
-
     _type = date
     default = date(1970, 1, 1)
 
@@ -104,7 +112,6 @@ class DateDimension(Dimension):
 
 
 class DateTimeDimension(Dimension):
-
     _type = datetime
     default = datetime(1970, 1, 1, 0, 0, 0)
 
@@ -120,7 +127,6 @@ class DateTimeDimension(Dimension):
 
 
 class ArrayDimension(Dimension):
-
     _type = list
     default = []
 
@@ -137,7 +143,6 @@ class ArrayDimension(Dimension):
 
 
 class MapDimension(Dimension):
-
     _type = dict
     default = {}
 
@@ -156,7 +161,6 @@ class MapDimension(Dimension):
 
 
 class IPDimension(Dimension):
-
     """Base for IP address dimensions"""
 
     _type = None
@@ -172,12 +176,10 @@ class IPDimension(Dimension):
 
 
 class IPv4Dimension(IPDimension):
-
     _type = ipaddress.IPv4Address
     default = "0.0.0.0"
 
 
 class IPv6Dimension(IPDimension):
-
     _type = ipaddress.IPv6Address
     default = "::"
