@@ -3,6 +3,7 @@ from typing import Union
 
 from hcube.api.models.aggregation import Aggregation
 from hcube.api.models.dimensions import Dimension
+from hcube.api.models.metrics import Metric
 
 
 class OrderDirection(Enum):
@@ -13,7 +14,7 @@ class OrderDirection(Enum):
 class OrderSpec:
     def __init__(
         self,
-        dimension: Union[Dimension, Aggregation],
+        dimension: Union[Dimension, Aggregation, Metric],
         direction: OrderDirection = OrderDirection.ASC,
     ):
         self.dimension = dimension
