@@ -61,7 +61,6 @@ def db_params_from_env(test=False):
 
 
 class PostgresCubeBackend(CubeBackend):
-
     """
     Backend to Postgres using the low-level Postgres API from `psycopg2`.
     """
@@ -109,7 +108,7 @@ class PostgresCubeBackend(CubeBackend):
     def _to_sql_dict(cls, rec):
         ret = {}
         for field, value in rec._asdict().items():
-            if type(value) is dict:
+            if type(value) is dict:  # noqa E721
                 ret[field] = Json(value)
             elif isinstance(value, (IPv4Address, IPv6Address)):
                 ret[field] = str(value)
@@ -134,10 +133,17 @@ class PostgresCubeBackend(CubeBackend):
 
     def delete_records(self, query: CubeQuery) -> None:
         # check that the query can be used
-        if query.aggregations or query.groups or query.transforms or query.orderings or query.limit:
+        if (
+            query.aggregations
+            or query.groups
+            or query.transforms
+            or query.orderings
+            or query.limit
+            or query.offset
+        ):
             raise ConfigurationError(
-                "Delete query can only have a filter, no aggregations, group_bys, ordering, limit "
-                "or transforms"
+                "Delete query can only have a filter, no aggregations, group_bys, ordering, limit,"
+                "offset or transforms"
             )
         query_parts = {}
         # we run it just for the parts and params
@@ -177,39 +183,49 @@ class PostgresCubeBackend(CubeBackend):
             for agg in query.aggregations:
                 agg_name = self._agg_name(agg.op)
                 inside = "*"
-                if agg.metric:
-                    inside = agg.metric.name
+                if agg.dim_or_metric:
+                    inside = agg.dim_or_metric.name
                 elif isinstance(agg, (Count, ArrayAgg)) and agg.distinct:
                     inside = f"DISTINCT {agg.distinct.name}"
                 # filters
                 filter_parts = []
                 filter_text = ""
                 for filtr in agg.filters:
-                    text, agg_params = self._ch_filter(filtr)
+                    text, agg_params = self._pg_filter(filtr)
                     filter_parts.append(text)
                     params.update(agg_params)
                 if filter_parts:
                     filter_text = "FILTER (WHERE " + " AND ".join(filter_parts) + ")"
+                # over
+                over = "" if not agg.over_all else "OVER ()"
                 # coalescing
                 if GlobalSettings.aggregates_zero_for_empty_data:
                     select_parts.append(
-                        f"COALESCE({agg_name}({inside}) {filter_text}, 0) AS {agg.name}"
+                        f"COALESCE({agg_name}({inside}) {filter_text}, 0) {over} AS {agg.name}"
                     )
                 else:
-                    select_parts.append(f"{agg_name}({inside}) {filter_text} AS {agg.name}")
+                    select_parts.append(f"{agg_name}({inside}) {filter_text} {over} AS {agg.name}")
                 fields.append(agg.name)
         else:
             fields = [dim.name for dim in query.cube._dimensions.values()] + [
                 metric.name for metric in query.cube._metrics.values()
             ]
             select_parts = fields[:]
+        # group aggregations
+        having_parts = []
+        for gf in query.group_filters:
+            inside = "*"
+            if gf.agg.dim_or_metric:
+                inside = gf.agg.dim_or_metric.name
+            fn_name = self._agg_name(gf.agg.op)
+            having_parts.append(f"{fn_name}({inside}) {gf.comparison.value} {gf.value}")
         select = ", ".join(select_parts)
         group_by = ", ".join(grp.name for grp in query.groups)
         table = f"{self.schema}.{self.cube_to_table_name(query.cube)}"
         order_by = ", ".join(f"{ob.dimension.name} {ob.direction.name}" for ob in query.orderings)
         where_parts = []
         for fltr in query.filters:
-            filter_text, filter_params = self._ch_filter(fltr)
+            filter_text, filter_params = self._pg_filter(fltr)
             where_parts.append(filter_text)
             params.update(filter_params)
         where = " AND ".join(where_parts)
@@ -219,11 +235,15 @@ class PostgresCubeBackend(CubeBackend):
             text += f"WHERE {where} "
         if group_by:
             text += f"GROUP BY {group_by} "
+        if having_parts:
+            text += f"HAVING {' AND '.join(having_parts)} "
         if order_by:
             text += f"ORDER BY {order_by} "
         if query.limit:
             text += f"LIMIT {query.limit} "
-        if type(parts) is dict:
+        if query.offset:
+            text += f"OFFSET {query.offset} "
+        if isinstance(parts, dict):
             parts.update(
                 {
                     "table": table,
@@ -236,7 +256,7 @@ class PostgresCubeBackend(CubeBackend):
             )
         return text, params, fields
 
-    def _ch_filter(self, fltr: Filter) -> (str, dict):
+    def _pg_filter(self, fltr: Filter) -> (str, dict):
         """
         returns a tuple with the string that should be put into the where part of the query and
         a dictionary with the parameters that should be passed to the query during execution
@@ -247,7 +267,7 @@ class PostgresCubeBackend(CubeBackend):
             queries = []
             params = {}
             for subfilter in fltr.filters:
-                query, subparams = self._ch_filter(subfilter)
+                query, subparams = self._pg_filter(subfilter)
                 queries.append(query)
                 params.update(subparams)
             return " OR ".join(queries), params
@@ -278,7 +298,12 @@ class PostgresCubeBackend(CubeBackend):
         raise ValueError(f"unsupported filter {fltr.__class__}")
 
     def _agg_name(self, agg: AggregationOp):
-        if agg in (AggregationOp.SUM, AggregationOp.COUNT, AggregationOp.MAX, AggregationOp.MIN):
+        if agg in (
+            AggregationOp.SUM,
+            AggregationOp.COUNT,
+            AggregationOp.MAX,
+            AggregationOp.MIN,
+        ):
             return agg.name
         if agg == AggregationOp.ARRAY:
             return "ARRAY_AGG"
